pub struct PlanarViewMut<'a, S: Sample, V: AsMut<[S]> + AsRef<[S]>> { /* private fields */ }Expand description
A mutable view of planar / separate-channel audio data.
- Layout:
[[ch0, ch0, ch0], [ch1, ch1, ch1]] - Interpretation: Each channel has its own separate buffer or array.
- Terminology: Also described as “planar” or “channels first” though more specifically it’s channel-isolated buffers.
- Usage: Very common in real-time DSP, as it simplifies memory access and can improve SIMD/vectorization efficiency.
Example
use audio_blocks::*;
let mut data = vec![[0.0, 0.0, 0.0], [1.0, 1.0, 1.0]];
let block = PlanarViewMut::from_slice(&mut data);
assert_eq!(block.channel(0), &[0.0, 0.0, 0.0]);
assert_eq!(block.channel(1), &[1.0, 1.0, 1.0]);Implementations
Sourceimpl<'a, S: Sample, V: AsMut<[S]> + AsRef<[S]>> PlanarViewMut<'a, S, V>
impl<'a, S: Sample, V: AsMut<[S]> + AsRef<[S]>> PlanarViewMut<'a, S, V>
Sourcepub fn from_slice(data: &'a mut [V]) -> Self
pub fn from_slice(data: &'a mut [V]) -> Self
Sourcepub fn from_slice_limited(
data: &'a mut [V],
num_channels_visible: u16,
num_frames_visible: usize,
) -> Self
pub fn from_slice_limited( data: &'a mut [V], num_channels_visible: u16, num_frames_visible: usize, ) -> Self
Creates a new audio block from a mutable slice with limited visibility.
This function allows creating a view that exposes only a subset of the allocated channels and frames, which is useful for working with a logical section of a larger buffer.
Parameters
data- The mutable slice containing planar audio samples (one slice per channel)num_channels_visible- Number of audio channels to expose in the viewnum_frames_visible- Number of audio frames to expose in the view
Panics
- Panics if
num_channels_visibleexceeds the number of channels indata - Panics if
num_frames_visibleexceeds the length of any channel buffer - Panics if channel slices have different lengths
Sourcepub fn channel_mut(&mut self, channel: u16) -> &mut [S]
pub fn channel_mut(&mut self, channel: u16) -> &mut [S]
Sourcepub fn channels(&self) -> impl ExactSizeIterator<Item = &[S]>
pub fn channels(&self) -> impl ExactSizeIterator<Item = &[S]>
Returns an iterator over all channels in the block.
Each channel is represented as a slice of samples.
Sourcepub fn channels_mut(&mut self) -> impl ExactSizeIterator<Item = &mut [S]>
pub fn channels_mut(&mut self) -> impl ExactSizeIterator<Item = &mut [S]>
Returns a mutable iterator over all channels in the block.
Each channel is represented as a mutable slice of samples.
Sourcepub fn raw_data(&self) -> &[V]
pub fn raw_data(&self) -> &[V]
Provides direct access to the underlying memory.
This function gives access to all allocated data, including any reserved capacity beyond the visible range.
Sourcepub fn raw_data_mut(&mut self) -> &mut [V]
pub fn raw_data_mut(&mut self) -> &mut [V]
Provides direct access to the underlying memory.
This function gives access to all allocated data, including any reserved capacity beyond the visible range.
pub fn view(&self) -> PlanarView<'_, S, V>
pub fn view_mut(&mut self) -> PlanarViewMut<'_, S, V>